﻿const express = require('express');
const { createClient } = require('@redis/client');
const { WebSocketServer } = require('ws');
const path = require('path');
const bodyParser = require('body-parser');

// Create an Express application
const app = express();
const port = 3000;

// Create a Redis client
const redisClient = createClient();

redisClient.on('error', (err) => console.error('Redis Client Error', err));

(async () => {
    await redisClient.connect();
})();

// Serve static files from the public directory
app.use(express.static(path.join(__dirname, 'public')));

// Create WebSocket server
const wss = new WebSocketServer({ port: 8080 });
let connectedClients = [];

// Handle WebSocket connections
wss.on('connection', (ws) => {
    console.log('New WebSocket connection');
    connectedClients.push(ws);

    ws.on('close', () => {
        connectedClients = connectedClients.filter(client => client !== ws);
        console.log('WebSocket disconnected');
    });
});

// Function to fetch data from Redis list using BLPOP (blocking pop)
async function fetchDataFromRedis() {
    try {

		const sendrealkey = 'QCS:11270016379255255255255';
        const result = await redisClient.blPop(sendrealkey, 0); // 0 means it will block until data is available
        if (result) {
            // result is [key, value] where value is the data we want
            const data = JSON.parse(result.element);
            return Array.isArray(data) ? data : [data]; // Ensure data is an array
        }
    } catch (err) {
        console.error('Error fetching data from Redis:', err);
    }
    return [];
}

// Polling Redis every second and sending data to WebSocket clients
setInterval(async () => {
    const processedData = await fetchDataFromRedis();
    console.log('Sending data:', processedData); // Log the data being sent
    connectedClients.forEach((client) => {
        if (client.readyState === client.OPEN) {
            client.send(JSON.stringify(processedData));
        }
    });
}, 1000); // 1 second interval

// Start the Express server
app.listen(port, () => {
    console.log(`HTTP Server is running at http://localhost:${port}`);
});

// Clean up on exit
process.on('SIGINT', async () => {
    console.log('Closing Redis client...');
    await redisClient.quit();
    process.exit(0);
});



// Redis 클라이언트 변수 선언
let besideredisClient;

app.use(bodyParser.json());
app.use(express.static('public')); // Static file serving

// Redis 연결 함수
const connectbesideredisClientRedis = () => {
    besideredisClient = createClient(); // Redis 클라이언트 생성
    besideredisClient.on('connect', () => console.log('Connected to Redis.'));
    besideredisClient.on('error', (err) => console.error('Redis error:', err));
    besideredisClient.connect(); // Redis 연결
};

// Redis 연결 초기화
connectbesideredisClientRedis();

app.post('/send', async (req, res) => {
    const sendData = req.body;
    console.log('Received data to send:', sendData);

    // sender 필드 출력
    console.log('Sender:', sendData.sender);
    
    try {
        const jsonText = JSON.stringify(sendData);

        // 키의 데이터 유형 확인
        const recvkey = 'CLK:' + sendData.sender;
        const type = await besideredisClient.type(recvkey);
        
        if (type !== 'none' && type !== 'list') {
            // 키가 존재하고 리스트가 아닌 경우 삭제
            await besideredisClient.del(recvkey);
        }

        // lPush로 데이터 전송
        await besideredisClient.lPush(recvkey, jsonText);
        console.log('Data sent to Redis:', jsonText);
        res.json({ message: 'Data sent successfully.' });
    } catch (error) {
        console.error('Failed to send data to Redis:', error);
        res.status(500).json({ error: 'Failed to send data.' });
    }
});

// 데이터 수신
app.get('/receive', async (req, res) => {
    console.log('Request received to fetch data from Redis');
    
    try {
        const sendkey = 'QCK:' + '11270016379255255255255';
        // Redis 클라이언트에서 blPop 명령 호출 (블로킹 팝)
        const result = await besideredisClient.blPop(sendkey, 0); // 0 means it will block until data is available
        
        if (result) {
            const data = JSON.parse(result.element); // result.element는 값
            return res.json(data); // 데이터를 JSON으로 반환
        }
    } catch (err) {
        console.error('Error fetching data from Redis:', err);
        return res.status(500).json({ error: 'Failed to fetch data.' });
    }
    
    return res.status(404).json({ message: 'No data found.' });
});
